@@ -1,5 +1,8 @@
 #!/usr/bin/env python
-"""This tool is automatic generate to pep8 checked code."""
+"""
+A tool that automatically formats Python code to conform to the PEP 8 style
+guide.
+"""
 import copy
 import os
 import re
@@ -12,9 +15,10 @@ import token
 import tokenize
 from optparse import OptionParser
 from subprocess import Popen, PIPE
-from difflib import unified_diff
+from difflib import unified_diff, SequenceMatcher
+import tempfile
 
-__version__ = '0.1.3'
+__version__ = '0.2'
 
 
 pep8bin = 'pep8'
@@ -54,6 +58,8 @@ class FixPEP8(object):
         self.results = []
         self.options = options
         self.indent_word = self._get_indentword("".join(self.source))
+        # method definition
+        self.fix_e222 = self.fix_e221
 
     def _find_newline(self, source):
         cr, lf, crlf = 0, 0, 0
@@ -136,7 +142,15 @@ class FixPEP8(object):
 
     def fix(self):
         pep8result = self._execute_pep8(self.filename)
-        self.results = [self._analyze_pep8result(line) for line in pep8result]
+        raw_results = [self._analyze_pep8result(line) for line in pep8result]
+
+        # Only handle one error per line
+        line_results = {}
+        for result in raw_results:
+            line_results[result['line']] = result
+
+        self.results = line_results.values()
+
         self._fix_source()
         return "".join(self.source)
 
@@ -151,15 +165,24 @@ class FixPEP8(object):
     #    self.source[result['line'] - 1] = fixed
 
     def fix_e201(self, result):
-        self._fix_whitespace(result, r"(\( )", "(")
+        self._fix_whitespace(result, r"\( ", "(")
+        self._fix_whitespace(result, r"\[ ", "[")
+        self._fix_whitespace(result, r"{ ", "{")
+
+    def fix_e202(self, result):
+        self._fix_whitespace(result, r" \)", ")")
+        self._fix_whitespace(result, r" \]", "]")
+        self._fix_whitespace(result, r" }", "}")
 
     def fix_e203(self, result):
-        self._fix_whitespace(result, r"(\) )", ")")
+        self._fix_whitespace(result, r" :", ":")
+        self._fix_whitespace(result, r" ,", ",")
 
     def fix_e211(self, result):
         self._fix_whitespace(result, r"( \()", "(")
 
     def fix_e221(self, result):
+        """e221 and e222 fixed method"""
         target = self.source[result['line'] - 1]
         c = result['column'] + 1
         fixed = re.sub(r'\s+', ' ', target[c::-1], 1)[::-1] + target[c + 1:]
@@ -168,16 +191,11 @@ class FixPEP8(object):
     def fix_e225(self, result):
         target = self.source[result['line'] - 1]
         offset = result['column']
-        fixed = ""
-        if target[offset - 2] not in tokenize.Whitespace:
-            fixed = target[:offset - 1] + " " + target[offset - 1]
-        else:
-            fixed = target[:offset - 1]
-        if target[offset] not in tokenize.Whitespace:
-            fixed += " " + target[offset:]
-        else:
-            fixed += target[offset:]
-        self.source[result['line'] - 1] = fixed
+        fixed = target[:offset - 1] + " " + target[offset - 1:]
+
+        # Only proceed if non-whitespace characters match
+        if fixed.replace(' ', '') == target.replace(' ', ''):
+            self.source[result['line'] - 1] = fixed
 
     def fix_e231(self, result):
         target = self.source[result['line'] - 1]
@@ -217,12 +235,16 @@ class FixPEP8(object):
     def fix_e303(self, result):
         delete_linenum = int(result['info'].split("(")[1].split(")")[0]) - 2
         for cnt in range(delete_linenum):
-            self.source[result['line'] - 2 - cnt] = ''
+            line = result['line'] - 2 - cnt
+            if not self.source[line].strip():
+                self.source[line] = ''
 
     def fix_e401(self, result):
         target = self.source[result['line'] - 1]
+        indentation = target.split("import ")[0]
         modules = target.split("import ")[1].split(",")
-        fixed_modulelist = ["import %s" % m.lstrip() for m in modules]
+        fixed_modulelist = \
+                [indentation + "import %s" % m.lstrip() for m in modules]
         self.source[result['line'] - 1] = self.newline.join(fixed_modulelist)
 
     def fix_e701(self, result):
@@ -234,10 +256,11 @@ class FixPEP8(object):
         self.source[result['line'] - 1] = fixed_source
 
     def fix_e702(self, result):
-        # FIXME: not fixing when multiple statements.
         target = self.source[result['line'] - 1]
-        f = target.split(";")
-        fixed = "".join(f)
+        non_whitespace_index = len(target) - len(target.lstrip())
+        indentation = target[:non_whitespace_index]
+        f = [indentation + t.lstrip() for t in target.split(";")]
+        fixed = '\n'.join(f)
         self.source[result['line'] - 1] = fixed
 
     def fix_w291(self, result):
@@ -268,7 +291,7 @@ class FixPEP8(object):
 
 
 def _get_difftext(old, new):
-    diff = unified_diff(old, new, 'oiginal', 'fixed')
+    diff = unified_diff(old, new, 'original', 'fixed')
     difftext = [line for line in diff]
     return "".join(difftext)
 
@@ -285,12 +308,27 @@ def main():
     if not len(args):
         print parser.format_help()
         return 1
-    fix = FixPEP8(args[0], opts)
+    filename = args[0]
+    tmp_source = open(filename).read()
+    fix = FixPEP8(filename, opts)
     fixed_source = fix.fix()
+    original_source = copy.copy(fix.original_source)
+    for cnt in range(5):
+        diff = SequenceMatcher(None, fixed_source, tmp_source)
+        if 1.0 == diff.ratio():
+            break
+        tmp_source = copy.copy(fixed_source)
+        filename = tempfile.mkstemp()[1]
+        fp = open(filename, 'w')
+        fp.write(fixed_source)
+        fp.close()
+        fix = FixPEP8(filename, opts)
+        fixed_source = fix.fix()
+        os.remove(filename)
     if opts.diff:
         new = StringIO("".join(fix.source))
         new = new.readlines()
-        print _get_difftext(fix.original_source, new),
+        print _get_difftext(original_source, new),
     else:
         print fixed_source,
 
